@@ -1,7 +1,7 @@
 /***************************************************************
  *                 Mathematical Object Library                 *
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 0.3.0                   *
+ *                   GNU/linux version 0.3.2                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright  2002,2005,2006 COLLARD Christophe
@@ -38,7 +38,7 @@
     \author copyright \htmlonly &#169; \endhtmlonly 2002, 2005, 2006 Christophe COLLARD \n
 	    copyright \htmlonly &#169; \endhtmlonly 2002, 2005, 2006 Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	    copyright \htmlonly &#169; \endhtmlonly 2002, 2005, 2006 Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 0.3.0
+    \version 0.3.2
     \date 2002-2006
     \bug none
     \warning none
@@ -56,8 +56,8 @@
 #include <assert.h>
 #endif
 
-#if !defined(__PRECISION_H)
-#include "precision.h"
+#if !defined(__PARAMETERS_H)
+#include "../parameters.h"
 #endif
 
 #if !defined(__MATH_H)
