@@ -2,7 +2,7 @@
  *                 Mathematical Object Library                 *
  *	      class matrix:  declarations for matrix	       *
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 1.9.12                  *
+ *	             GNU/linux version 1.9.30                  *
  *            software under General Public License            *
  ***************************************************************
  * copyright  1995,1996,1999,2000,2001,2002,2003,2004,2005,2006 COLLARD Christophe
@@ -43,7 +43,7 @@
 	     copyright \htmlonly &#169; \endhtmlonly 2003, 2004 Denis MERCIER \n
 	     copyright \htmlonly &#169; \endhtmlonly 2002, 2003, 2004, 2005, 2006 Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	     copyright \htmlonly &#169; \endhtmlonly 2002, 2003, 2004, 2005, 2006 Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 1.9.12
+    \version 1.9.30
     \date 1995-2006
     \bug none
     \warning none
@@ -73,6 +73,10 @@
 #include <math.h>
 #endif
 
+#if !defined(__PARAMETERS_H)
+#include "../parameters.h"
+#endif
+
 #if !defined(__MATHS_H)
 #include "maths.h"
 #endif
@@ -107,55 +111,57 @@ template <class T> class matrix
     virtual int Columns () const {return nbColumns;}     // returns the # of columns
     virtual void create (int,int);  // partial memory allocation (don't use this function except if you know what you're doing !!)
     virtual void assign (int,int);  // allocates memory without initilizing the matrix components to zero (so use it with great care)
+    operator int () const {return nbRows;}  // cast operator : matrix -> int
+    friend bool operator ! (const matrix<T>& a) {return !a.nbRows;}
 
     virtual T&   operator () (int,int) const;
-    virtual vector<T>& operator [] (int) const;            // returns a whole row
-    virtual matrix<T>& operator = (const matrix<T>&);      // defines equal operator for matrix
-    virtual matrix<T>& operator = (matrix<T>*);  // allocates the data from the right hand temporary object to the left hand object - no copy (deletes temporary object)
-    virtual matrix<T>& operator &= (matrix<T>&); // allocates the data from the right hand member to the left one (deletes right hand member) - no size check - no copy (use this operator with great care)
-    virtual matrix<T>& operator &= (matrix<T>*); // allocates the data from the right hand temporary object to the left hand object (deletes temporary object) - no size check - no copy (use this operator with great care)
-    template <class M> friend matrix<M> operator + (const matrix<M>&, const matrix<M>&); // overloads plus operator for matrix
-    template <class M> friend matrix<M> operator - (const matrix<M>&, const matrix<M>&); // overloads minus operator for matrix
-    template <class M> friend matrix<M> operator * (const matrix<M>&, const matrix<M>&); // overloads multiplicate operator for matrix or tensorial product (A.B)ij = Aik Bkj = Tij
-    template <class M> friend matrix<M> operator * (const M&, const matrix<M>&);         // overloads multiplication between scalar & matrix
-    template <class M> friend matrix<M> operator * (const matrix<M>&, const M&);         // overloads multiplication between matrix & scalar
-    template <class M> friend vector<M> operator * (const matrix<M>&, const vector<M>&); // overloads multiplication between matrix & vector
-    template <class M> friend vector<M> operator * (const vector<M>&, const matrix<M>&); // overloads multiplication between vector & matrix
-    template <class M> friend matrix<M> operator / (const matrix<M>&, const matrix<M>&); // overloads division operator for matrix
-    template <class M> friend matrix<M> operator / (const M&, const matrix<M>&);         // overloads division between scalar & matrix
-    template <class M> friend matrix<M> operator / (const matrix<M>&, const M&);         // overloads division between scalar & matrix
-    template <class M> friend M         operator | (const matrix<M>&, const matrix<M>&); // A|B = Aij Bij (for matrix & tensor2)
-    template <class M> friend matrix<M> operator ^ (const vector<M>&, const vector<M>&); // overloads operator ^ for matrix = vector*t(vector) (tensorial product for 2 vectors)
-    void operator += (const matrix<T>&);                                    // overloads += operator for matrix
-    void operator -= (const matrix<T>&);                                    // overloads -= operator for matrix
-    void operator *= (const matrix<T>&);                                    // overloads *= operator for matrix
-    void operator *= (const T&);                                            // overloads *= operator between matrix and scalar
-    void operator /= (const matrix<T>&);                                    // overloads /= operator for matrix
-    void operator /= (const T&);	                                    // overloads /= operator between matrix and scalar
-    friend bool operator ! (const matrix<T>& a) {return !a.nbRows;}
-    template <class M> friend bool operator == (const matrix<M>&, const matrix<M>&);  // compares 2 matrices
-    template <class M> friend bool operator != (const matrix<M>&, const matrix<M>&);
-    template <class M> friend ostream& operator << (ostream&, const matrix<M>&);      // overloads output stream for matrix
-    template <class M> friend istream& operator >> (istream&, const matrix<M>&);      // overloads input stream for matrix
-    template <class M> friend M det (const matrix<M>&);                     // computes the determinant operator for a matrix
-    template <class M> friend matrix<M> t (const matrix<M>&);               // defines tranposition operator for matrix
-    virtual matrix<T> inv ();	                                            // computes inversion operator for matrix
-    template <class M> friend matrix<M> gauss (matrix<M>);                  // computes matrix inversion with gauss algorithm
-    template <class M> friend int inv (matrix<M>&);                         // same function as gauss but returns the inversion result 0/1
-    //    friend matrix<T> gauss_pt <T> (matrix<T>);                              // computes matrix inversion with gauss algorithm
-    virtual void save (const char*); // writes to disk
-    virtual void read (const char*); // reads from disk
-    //    friend matrix<T> kronecker <T> (int); // return the identity matrix (ie. i==j -> 1 and i!=j -> 0)
+    virtual vector<T>& operator [] (int) const;        // returns a whole row
+    virtual matrix<T>& operator = (const matrix<T>&);  // defines equal operator for matrix
+    virtual matrix<T>& operator = (matrix<T>*);        // allocates the data from the right hand temporary object to the left hand object - no copy (deletes temporary object)
+    virtual matrix<T>& operator &= (const matrix<T>&); // copies matrix without size check (use this operator with great care)
+    virtual matrix<T>& operator &= (matrix<T>*);       // allocates the data from the right hand temporary object to the left hand object (deletes temporary object) - no size check - no copy (use this operator with great care)
+    template <class Tf> friend matrix<Tf> operator + (const matrix<Tf>&, const matrix<Tf>&); // overloads matrix + matrix
+    template <class Tf> friend matrix<Tf> operator - (const matrix<Tf>&, const matrix<Tf>&); // overloads matrix - matrix
+    template <class Tf> friend matrix<Tf> operator * (const matrix<Tf>&, const matrix<Tf>&); // overloads matrix * matrix
+    template <class Tf> friend matrix<Tf> operator * (const Tf&, const matrix<Tf>&);         // overloads scalar * matrix
+    template <class Tf> friend matrix<Tf> operator * (const matrix<Tf>&, const Tf&);         // overloads matrix * scalar
+    template <class Tf> friend vector<Tf> operator * (const matrix<Tf>&, const vector<Tf>&); // overloads matrix * vector
+    template <class Tf> friend vector<Tf> operator * (const vector<Tf>&, const matrix<Tf>&); // overloads vector * matrix
+    template <class Tf> friend matrix<Tf> operator / (const matrix<Tf>&, const matrix<Tf>&); // overloads matrix / matrix
+    template <class Tf> friend matrix<Tf> operator / (const Tf&, const matrix<Tf>&);         // overloads scalar / matrix
+    template <class Tf> friend matrix<Tf> operator / (const matrix<Tf>&, const Tf&);         // overloads scalar / matrix
+    template <class Tf> friend Tf         operator | (const matrix<Tf>&, const matrix<Tf>&); // A|B = Aij Bij (for matrix & tensor2)
+    template <class Tf> friend matrix<Tf> operator ^ (const vector<Tf>&, const vector<Tf>&); // overloads operator ^ for matrix = vector*t(vector) (tensorial product for 2 vectors)
+    void operator += (const matrix<T>&);  // overloads += operator for matrix
+    void operator -= (const matrix<T>&);  // overloads -= operator for matrix
+    void operator *= (const matrix<T>&);  // overloads *= operator for matrix
+    void operator *= (const T&);          // overloads *= operator between matrix and scalar
+    void operator /= (const matrix<T>&);  // overloads /= operator for matrix
+    void operator /= (const T&);	  // overloads /= operator between matrix and scalar
+    template <class Tf> friend bool operator == (const matrix<Tf>&, const matrix<Tf>&);  // compares 2 matrices
+    template <class Tf> friend bool operator != (const matrix<Tf>&, const matrix<Tf>&);
+    template <class Tf> friend ostream& operator << (ostream&, const matrix<Tf>&);       // overloads output stream for matrix
+    template <class Tf> friend istream& operator >> (istream&, const matrix<Tf>&);       // overloads input stream for matrix
+    template <class Tf> friend matrix<Tf> Id2 (int);              // returns identity matrix
+    template <class Tf> friend Tf det (const matrix<Tf>&);        // computes the determinant operator for a matrix
+    template <class Tf> friend matrix<Tf> t (const matrix<Tf>&);  // defines tranposition operator for matrix
+    virtual matrix<T> inv ();	                                  // computes inversion operator for matrix
+    template <class Tf> friend matrix<Tf> gauss (matrix<Tf>);     // computes matrix inversion with gauss algorithm
+    template <class Tf> friend bool inv (matrix<Tf>&);            // same function as gauss but returns the inversion result 0/1
+    //    friend matrix<T> gauss_pt <T> (matrix<T>);              // computes matrix inversion with gauss algorithm
+    virtual void save (const char*);  // writes to disk
+    virtual void read (const char*);  // reads from disk
+    template <class Tf> friend matrix<Tf> kronecker (int); // return the identity matrix (ie. i==j -> 1 and i!=j -> 0)
     matrix<T>& approximation ();
-    virtual void permute_rows (int,int); // permutes matrix rows
-    virtual void permute_columns (int,int); // permutes matrix columns
-    template <class M> friend matrix<M> abs (const matrix<M>&);  // gives the absolute value of a matrix
-    template <class M> friend M max (const matrix<M>&);  // gives the greatest element of a matrix
-    template <class M> friend M tr (const matrix<M>&);   // gives the trace of a matrix
-    template <class M> friend vector<M> diagonal (const matrix<M>&);
-    template <class M> friend matrix<M> submatrix (const matrix<M>&, int, int, int, int);
-    template <class M> friend matrix<M> merge_rows (const matrix<M>&, const matrix<M>&);
-    template <class M> friend matrix<M> merge_columns (const matrix<M>&, const matrix<M>&);
+    virtual void permute_rows (int,int);     // permutes matrix rows
+    virtual void permute_columns (int,int);  // permutes matrix columns
+    template <class Tf> friend matrix<Tf> abs (const matrix<Tf>&);  // gives the absolute value of a matrix
+    template <class Tf> friend Tf max (const matrix<Tf>&);  // gives the greatest element of a matrix
+    template <class Tf> friend Tf tr (const matrix<Tf>&);   // gives the trace of a matrix
+    template <class Tf> friend vector<Tf> diagonal (const matrix<Tf>&);
+    template <class Tf> friend matrix<Tf> submatrix (const matrix<Tf>&, int, int, int, int);
+    template <class Tf> friend matrix<Tf> merge_rows (const matrix<Tf>&, const matrix<Tf>&);
+    template <class Tf> friend matrix<Tf> merge_columns (const matrix<Tf>&, const matrix<Tf>&);
 };
 
 
@@ -312,19 +318,24 @@ template <class T> matrix<T>& matrix<T>::operator = (matrix<T>* mat)
 }
 
 
-//-------------------------------------------------------------------
-template <class T> matrix<T>& matrix<T>::operator &= (matrix<T>& mat)
-//-------------------------------------------------------------------
+//-------------------------------------------------------------------------
+template <class T> matrix<T>& matrix<T>::operator &= (const matrix<T>& mat)
+//-------------------------------------------------------------------------
 {
-  if (!mat.nbRows) nbRows = nbColumns = 0;
+  if (!mat)
+    { if (nbRows) delete [] mrows;
+      nbRows = nbColumns = 0;
+    }
+
   else
-    { assert (mat[1].dim() == mat.nbColumns); // forbids the acces to symmetric matrices
-      if (nbRows)
-	delete [] mrows;
-      nbRows = mat.nbRows;
+    { assert (mat[1].dim() == mat.nbColumns); // forbids the acces to matrix & spmatrix
+      if (nbRows != mat.nbRows)
+	{ if (nbRows) delete [] mrows;
+	  mrows = new vector<T> [nbRows = mat.nbRows];  // allocates memory with the = operator for vector (no initialization)
+	}
       nbColumns = mat.nbColumns;
-      mrows = mat.mrows;
-      mat.nbRows = mat.nbColumns = 0;
+      for (int i=1; i<=mat.nbRows; i++)
+	(*this)[i] &= mat[i];
     }
 
   return *this;
@@ -335,11 +346,14 @@ template <class T> matrix<T>& matrix<T>::operator &= (matrix<T>& mat)
 template <class T> matrix<T>& matrix<T>::operator &= (matrix<T>* mat)
 //-------------------------------------------------------------------
 {
-  if (!(*mat).nbRows) nbRows = nbColumns = 0;
+  if (!(*mat).nbRows)
+    { if (nbRows) delete [] mrows;
+      nbRows = nbColumns = 0;
+    }
+
   else
-    { assert ((*mat)[1].dim() == (*mat).nbColumns); // forbids the acces to symmetric matrices
-      if (nbRows)
-	delete [] mrows;
+    { assert ((*mat)[1].dim() == (*mat).nbColumns); // forbids the acces to symmetric matrices in order to avoid matrix &= symmatrix
+      if (nbRows) delete [] mrows;
       nbRows = (*mat).nbRows;
       nbColumns = (*mat).nbColumns;
       mrows = (*mat).mrows;
@@ -350,41 +364,41 @@ template <class T> matrix<T>& matrix<T>::operator &= (matrix<T>* mat)
 }
 
 
-//------------------------------------------------------------------------------------
-template <class T> matrix<T> operator + (const matrix<T>& mat1, const matrix<T>& mat2)
-//------------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------------
+template <class Tf> matrix<Tf> operator + (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
+//----------------------------------------------------------------------------------------
 {
   assert ((mat1.nbRows==mat2.nbRows) && (mat1.nbColumns==mat2.nbColumns)); // matrices must have the same size
   assert((mat1.nbRows)&&(mat1.nbColumns));
 
-  matrix<T> mat = mat1;
+  matrix<Tf> mat = mat1;
   mat += mat2;
 
   return mat;
 }
 
 
-//------------------------------------------------------------------------------------
-template <class T> matrix<T> operator - (const matrix<T>& mat1, const matrix<T>& mat2)
-//------------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------------
+template <class Tf> matrix<Tf> operator - (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
+//----------------------------------------------------------------------------------------
 {
   assert ((mat1.nbRows==mat2.nbRows) && (mat1.nbColumns==mat2.nbColumns)); // matrices must have the same size
   assert((mat1.nbRows)&&(mat1.nbColumns));
 
-  matrix<T> mat = mat1;
+  matrix<Tf> mat = mat1;
   mat -= mat2;
 
   return mat;
 }
 
 
-//------------------------------------------------------------------------------------
-template <class T> matrix<T> operator * (const matrix<T>& mat1, const matrix<T>& mat2)
-//------------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------------
+template <class Tf> matrix<Tf> operator * (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
+//----------------------------------------------------------------------------------------
 {
   assert (mat1.nbColumns==mat2.nbRows); // matrix can't be multiplicated
 
-  matrix<T> mt(mat1.nbRows,mat2.nbColumns); // allocates memory and set matrix components to zero
+  matrix<Tf> mt(mat1.nbRows,mat2.nbColumns); // allocates memory and set matrix components to zero
 
   for (int i=1; i<=mat1.nbRows; i++)
     for (int j=1; j<=mat2.nbColumns; j++)
@@ -395,34 +409,34 @@ template <class T> matrix<T> operator * (const matrix<T>& mat1, const matrix<T>&
 }
 
 
-//--------------------------------------------------------------------------
-template <class T> matrix<T> operator * (const matrix<T>& mat, const T& elt)
-//--------------------------------------------------------------------------
+//------------------------------------------------------------------------------
+template <class Tf> matrix<Tf> operator * (const matrix<Tf>& mat, const Tf& elt)
+//------------------------------------------------------------------------------
 {
   assert (mat.nbRows && mat.nbColumns);
-  matrix<T> mp = mat;
+  matrix<Tf> mp = mat;
   mp *= elt;
   return mp;
 }
 
 
-//--------------------------------------------------------------------------
-template <class T> matrix<T> operator * (const T& elt, const matrix<T>& mat)
-//--------------------------------------------------------------------------
+//------------------------------------------------------------------------------
+template <class Tf> matrix<Tf> operator * (const Tf& elt, const matrix<Tf>& mat)
+//------------------------------------------------------------------------------
 {
   assert (mat.nbRows&&mat.nbColumns);
-  matrix<T> mp = mat;
+  matrix<Tf> mp = mat;
   mp *= elt;
   return mp;
 }
 
 
-//--------------------------------------------------------------------------------
-template <class T> vector<T> operator * (const matrix<T>& mat, const vector<T>& v)
-//--------------------------------------------------------------------------------
+//------------------------------------------------------------------------------------
+template <class Tf> vector<Tf> operator * (const matrix<Tf>& mat, const vector<Tf>& v)
+//------------------------------------------------------------------------------------
 { 
   assert (mat.nbColumns == v.dim()); // checks validity of the operation
-  vector<T> u(mat.nbRows); // allocates memory and set vector components to zero
+  vector<Tf> u(mat.nbRows); // allocates memory and set vector components to zero
   for (int i=1; i<=mat.nbRows; i++)
       for (int j=1; j<=mat.nbColumns;j++)
            u[i] += mat(i,j) * v[j];
@@ -430,12 +444,12 @@ template <class T> vector<T> operator * (const matrix<T>& mat, const vector<T>&
 }
 
 
-//--------------------------------------------------------------------------------
-template <class T> vector<T> operator * (const vector<T>& v, const matrix<T>& mat)
-//--------------------------------------------------------------------------------
+//------------------------------------------------------------------------------------
+template <class Tf> vector<Tf> operator * (const vector<Tf>& v, const matrix<Tf>& mat)
+//------------------------------------------------------------------------------------
 { 
   assert (mat.nbRows == v.dim());
-  vector<T> u(mat.nbColumns); // allocates memory and set vector components to zero
+  vector<Tf> u(mat.nbColumns); // allocates memory and set vector components to zero
   for (int i=1; i<=mat.nbColumns; i++)
       for (int j=1; j<=mat.nbRows; j++)
           u[i] += v[j] * mat(j,i);
@@ -443,45 +457,45 @@ template <class T> vector<T> operator * (const vector<T>& v, const matrix<T>& ma
 }
 
 
-//------------------------------------------------------------------------------------
-template <class T> matrix<T> operator / (const matrix<T>& mat1, const matrix<T>& mat2)
-//------------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------------
+template <class Tf> matrix<Tf> operator / (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
+//----------------------------------------------------------------------------------------
 {
-  matrix<T> mt = mat1;
+  matrix<Tf> mt = mat1;
   mt *= gauss(mat2);
   return mt;
 }
 
 
-//--------------------------------------------------------------------------
-template <class T> matrix<T> operator / (const T& elt, const matrix<T>& mat)
-//--------------------------------------------------------------------------
+//------------------------------------------------------------------------------
+template <class Tf> matrix<Tf> operator / (const Tf& elt, const matrix<Tf>& mat)
+//------------------------------------------------------------------------------
 {
-  matrix<T> mt = gauss(mat);
+  matrix<Tf> mt = gauss(mat);
   mt*= elt;
   return mt;
 }
 
 
-//--------------------------------------------------------------------------
-template <class T> matrix<T> operator / (const matrix<T>& mat, const T& elt)
-//--------------------------------------------------------------------------
+//------------------------------------------------------------------------------
+template <class Tf> matrix<Tf> operator / (const matrix<Tf>& mat, const Tf& elt)
+//------------------------------------------------------------------------------
 {
   assert (elt!=0); // can't divide by zero !
-  matrix<T> mp = mat;
+  matrix<Tf> mp = mat;
   mp /= elt;
   return mp;
 }
 
 
-//----------------------------------------------------------------------------
-template <class T> T operator | (const matrix<T>& mat1, const matrix<T>& mat2)
-//----------------------------------------------------------------------------
+//--------------------------------------------------------------------------------
+template <class Tf> Tf operator | (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
+//--------------------------------------------------------------------------------
 {
   assert (mat1.nbRows==mat2.nbRows); // matrices must have the same size
   assert (mat1.nbRows);
 
-  T sum=0;
+  Tf sum=0;
   for (int i=1; i<=mat1.nbRows; i++)
     for (int j=1; j<=mat1.nbColumns; j++)
       sum += mat1(i,j) * mat2(i,j);
@@ -490,12 +504,12 @@ template <class T> T operator | (const matrix<T>& mat1, const matrix<T>& mat2)
 }
 
 
-//------------------------------------------------------------------------------
-template <class T> matrix<T> operator ^ (const vector<T>& v, const vector<T>& w)
-//------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------
+template <class Tf> matrix<Tf> operator ^ (const vector<Tf>& v, const vector<Tf>& w)
+//----------------------------------------------------------------------------------
 {
   assert (v.dim() && w.dim());
-  matrix<T> mat(v.dim(),w.dim(),false); // allocates memory for columns (no initialization)
+  matrix<Tf> mat(v.dim(),w.dim(),false); // allocates memory for columns (no initialization)
   for (int i=1; i<=v.dim(); i++)
     for (int j=1; j<=w.dim(); j++)
       mat (i,j) = v[i]*w[j];
@@ -533,7 +547,7 @@ template <class T> void matrix<T>::operator -= (const matrix<T>& mat)
 template <class T> void matrix<T>::operator *= (const matrix<T>& mat)
 //-------------------------------------------------------------------
 {
-  assert ((nbColumns==mat.nbRows)&&(nbColumns==mat.nbColumns)); // A(x,y) * B(y,y)
+  assert ((nbColumns==mat.nbRows)&&(nbColumns==mat.nbColumns));
   assert (nbColumns==(*this)[1].dim()); // this operator is not defined for symmatrix on the left side of the equation
   matrix<T> mt(nbRows,nbColumns); // allocates memory and set matrix components to zero
 
@@ -563,6 +577,7 @@ template <class T> void matrix<T>::operator /= (const matrix<T>& mat)
 //-------------------------------------------------------------------
 {
   // a modifier et optimiser - cf operateur /
+  assert (nbColumns==(*this)[1].dim()); // this operator is not defined for symmatrix on the left side of the equation
 
   *this *= gauss(mat);
 }
@@ -579,9 +594,9 @@ template <class T> void matrix<T>::operator /= (const T& elt)
 }
 
 
-//--------------------------------------------------------------------------------
-template <class T> bool operator == (const matrix<T>& mat1, const matrix<T>& mat2)
-//--------------------------------------------------------------------------------
+//-----------------------------------------------------------------------------------
+template <class Tf> bool operator == (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
+//-----------------------------------------------------------------------------------
 {
   bool result=( (mat1.nbRows==mat2.nbRows) && (mat2.nbColumns==mat2.nbColumns) );
 
@@ -593,17 +608,17 @@ template <class T> bool operator == (const matrix<T>& mat1, const matrix<T>& mat
 }
 
 
-//--------------------------------------------------------------------------------
-template <class T> bool operator != (const matrix<T>& mat1, const matrix<T>& mat2)
-//--------------------------------------------------------------------------------
+//-----------------------------------------------------------------------------------
+template <class Tf> bool operator != (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
+//-----------------------------------------------------------------------------------
 {
   return !(mat1 == mat2);
 }
 
 
-//------------------------------------------------------------------------
-template <class T> ostream& operator << (ostream& s, const matrix<T>& mat)
-//------------------------------------------------------------------------
+//--------------------------------------------------------------------------
+template <class Tf> ostream& operator << (ostream& s, const matrix<Tf>& mat)
+//--------------------------------------------------------------------------
 {
   for (int i=1; i<=mat.nbRows; i++)
       s << mat[i];
@@ -612,9 +627,9 @@ template <class T> ostream& operator << (ostream& s, const matrix<T>& mat)
 }
 
 
-//------------------------------------------------------------------------
-template <class T> istream& operator >> (istream& s, const matrix<T>& mat)
-//------------------------------------------------------------------------
+//--------------------------------------------------------------------------
+template <class Tf> istream& operator >> (istream& s, const matrix<Tf>& mat)
+//--------------------------------------------------------------------------
 {
   for (int i=1; i<=mat.nbRows; i++)
       s >> mat[i];
@@ -622,14 +637,28 @@ template <class T> istream& operator >> (istream& s, const matrix<T>& mat)
 }
 
 
-//---------------------------------------------
-template <class T> T det (const matrix<T>& mat)
-//---------------------------------------------
+//----------------------------------------
+template <class Tf> matrix<Tf> Id2 (int n)
+//----------------------------------------
+{
+  assert (n > 0);
+  matrix<Tf> I(n,n);
+
+  for (int i=1; i<=n; i++)
+    I(i,i) = 1;
+
+  return I;
+}
+
+
+//------------------------------------------------
+template <class Tf> Tf det (const matrix<Tf>& mat)
+//------------------------------------------------
 {
   assert (mat.nbRows == mat.nbColumns); // matrix must be squared
-  T value = 0;
+  Tf value = 0;
   if (mat.nbRows==1) return mat(1,1);
-  matrix<T> m(mat.nbRows-1,mat.nbColumns-1);
+  matrix<Tf> m(mat.nbRows-1,mat.nbColumns-1);
 
   for (int i=1; i<=mat.nbRows; i++)
   {   for (int j=2; j<=mat.nbColumns; j++)
@@ -648,12 +677,12 @@ template <class T> T det (const matrix<T>& mat)
 }
 
 
-//---------------------------------------------------
-template <class T> matrix<T> t (const matrix<T>& mat)
-//---------------------------------------------------
+//------------------------------------------------------
+template <class Tf> matrix<Tf> t (const matrix<Tf>& mat)
+//------------------------------------------------------
 {
   assert (mat.nbRows && mat.nbColumns);
-  matrix<T> mt(mat.nbColumns,mat.nbRows,false); // allocates memory (no initialization)
+  matrix<Tf> mt(mat.nbColumns,mat.nbRows,false); // allocates memory (no initialization)
   for (int i=1; i<=mat.nbRows; i++)
       for (int j=1; j<=mat.nbColumns; j++)
 	  mt(j,i) = mat(i,j);
@@ -693,16 +722,14 @@ template <class T> matrix<T> matrix<T>::inv ()
 }
 
 
-//------------------------------------------------
-template <class T> matrix<T> gauss (matrix<T> mat)
-//------------------------------------------------
+//---------------------------------------------------
+template <class Tf> matrix<Tf> gauss (matrix<Tf> mat)
+//---------------------------------------------------
 {
   assert (mat.nbRows == mat.nbColumns); // matrix must be squared
-  matrix <T> matinv (mat.nbRows, mat.nbColumns); // allocates memory and set matrix components to zero
+  matrix<Tf> matinv =& Id2<Tf>(mat.nbRows); // initializes matinv as the identity matrix
   int posx, dim = mat.nbRows;
-  T pivot, elt;
-
-  for (int i=1; i<=dim; i++) matinv(i,i) = 1; // initialises matinv (matinv - Id)
+  Tf pivot, elt;
 
   for (int k=1; k<=dim; k++)
     { pivot = 0;
@@ -722,13 +749,13 @@ template <class T> matrix<T> gauss (matrix<T> mat)
       	}
 
       //  starts the resolution at step k
-      vector<T> *mat_k, *matinv_k;
+      vector<Tf> *mat_k, *matinv_k;
       mat_k = &mat[k];
       matinv_k = &matinv[k];
-      T mat_kk = (*mat_k)[k];
+      Tf mat_kk = (*mat_k)[k];
 
       for (int i=1; i<k; i++)
-      	{ vector<T> *mat_i, *matinv_i;
+      	{ vector<Tf> *mat_i, *matinv_i;
 	  mat_i = &mat[i];
 	  matinv_i = &matinv[i];
 	  elt = (*mat_i)[k] / mat_kk;
@@ -736,7 +763,7 @@ template <class T> matrix<T> gauss (matrix<T> mat)
       	  for (int j=1; j<=dim; j++) (*matinv_i)[j] -= (*matinv_k)[j] * elt;
 	}
       for (int i=k+1; i<=dim; i++)
-	{ vector<T> *mat_i, *matinv_i;
+	{ vector<Tf> *mat_i, *matinv_i;
 	  mat_i = &mat[i];
 	  matinv_i = &matinv[i];
 	  elt = (*mat_i)[k] / mat_kk;
@@ -751,16 +778,15 @@ template <class T> matrix<T> gauss (matrix<T> mat)
 }
 
 
-//-----------------------------------------
-template <class T> int inv (matrix<T>& mat)
-//-----------------------------------------
+//--------------------------------------------
+template <class Tf> bool inv (matrix<Tf>& mat)
+//--------------------------------------------
 {
   assert (mat.nbRows == mat.nbColumns); // matrix must be squared
-  matrix <T> matinv (mat.nbRows, mat.nbColumns); // allocates memory and set matrix components to zero
+  matrix <Tf> matinv =& Id2<Tf>(mat.nbRows); // initializes matinv as the identity matrix
+  matrix<Tf> mat_copy = mat;
   int posx, dim = mat.nbRows;
-  T pivot, elt;
-
-  for (int i=1; i<=dim; i++) matinv(i,i) = 1; // initialises matinv (matinv - Id)
+  Tf pivot, elt;
 
   for (int k=1; k<=dim; k++)
     { pivot = 0;
@@ -771,13 +797,10 @@ template <class T> int inv (matrix<T>& mat)
 	    posx = i;
 	  }
 
-      if (abs(pivot) < epsilon) return 0;
-
-      // Rows permutation : row k <-> row posx
-      if (posx != k)
-      	{ mat.permute_rows(k,posx);
-      	  matinv.permute_rows(k,posx);
-      	}
+      if (abs(pivot) < epsilon)
+	{ mat = mat_copy;
+	  return false;
+	}
 
       // Rows permutation : row k <-> row posx
       if (posx != k)
@@ -786,13 +809,13 @@ template <class T> int inv (matrix<T>& mat)
       	}
 
       //  starts the resolution at step k
-      vector<T> *mat_k, *matinv_k;
+      vector<Tf> *mat_k, *matinv_k;
       mat_k = &mat[k];
       matinv_k = &matinv[k];
-      T mat_kk = (*mat_k)[k];
+      Tf mat_kk = (*mat_k)[k];
 
       for (int i=1; i<k; i++)
-      	{ vector<T> *mat_i, *matinv_i;
+      	{ vector<Tf> *mat_i, *matinv_i;
 	  mat_i = &mat[i];
 	  matinv_i = &matinv[i];
 	  elt = (*mat_i)[k] / mat_kk;
@@ -800,7 +823,7 @@ template <class T> int inv (matrix<T>& mat)
       	  for (int j=1; j<=dim; j++) (*matinv_i)[j] -= (*matinv_k)[j] * elt;
 	}
       for (int i=k+1; i<=dim; i++)
-	{ vector<T> *mat_i, *matinv_i;
+	{ vector<Tf> *mat_i, *matinv_i;
 	  mat_i = &mat[i];
 	  matinv_i = &matinv[i];
 	  elt = (*mat_i)[k] / mat_kk;
@@ -812,7 +835,7 @@ template <class T> int inv (matrix<T>& mat)
     }
 
   mat =& matinv;
-  return 1;
+  return true;
 }
 
 /* FAUX - A REVOIR
@@ -821,12 +844,10 @@ template <class T> matrix<T> gauss_pt (matrix<T> mat)
 //---------------------------------------------------
 {
   assert (mat.nbRows == mat.nbColumns); // matrix must be squared
-  matrix <T> matinv (mat.nbRows, mat.nbColumns); // allocates memory and set matrix components to zero
+  matrix <T> matinv =& Id2<T>(mat.nbRows); // initializes matinv as the identity matrix
   int posx, posy, dim = mat.nbRows;
   T pivot, elt;
 
-  for (int i=1; i<=dim; i++) matinv(i,i) = 1; // initialises matinv (matinv - Id)
-
   for (int k=1; k<=dim; k++)
     { pivot = 0;
       // search the biggest element of the ith line and kth column
@@ -903,14 +924,12 @@ template <class T> matrix<T>& matrix<T>::approximation ()
 }
 
 
-//--------------------------------------------
-template <class T> matrix<T> kronecker (int n)
-//--------------------------------------------
+//----------------------------------------------
+template <class Tf> matrix<Tf> kronecker (int n)
+//----------------------------------------------
 {
-  matrix<T> mat(n,n); // allocates memory and set matrix components to zero
-  for (int i=1; i<=n; i++)
-    mat(i,i) = 1;
-  return mat;
+  assert (n > 0);
+  return Id2<Tf>(n);
 }
 
 
@@ -944,12 +963,12 @@ template <class T> void matrix<T>::permute_columns (int i, int j)
 }
 
 
-//-----------------------------------------------------
-template <class T> matrix<T> abs (const matrix<T>& mat)
-//-----------------------------------------------------
+//--------------------------------------------------------
+template <class Tf> matrix<Tf> abs (const matrix<Tf>& mat)
+//--------------------------------------------------------
 {
   assert (mat.nbColumns);
-  matrix<T> mtx;
+  matrix<Tf> mtx;
   mtx.create(mat.nbRows,mat.nbColumns);
 
   for (int i=1; i<=mat.nbRows; i++)
@@ -959,12 +978,12 @@ template <class T> matrix<T> abs (const matrix<T>& mat)
 }
 
 
-//---------------------------------------------
-template <class T> T max (const matrix<T>& mat)
-//---------------------------------------------
+//------------------------------------------------
+template <class Tf> Tf max (const matrix<Tf>& mat)
+//------------------------------------------------
 {
   assert (mat.nbColumns);
-  T val = mat(1,1);
+  Tf val = mat(1,1);
 
   for (int i=1; i<=mat.nbRows; i++)
     for (int j=1; j<=mat.nbColumns; j++)
@@ -974,14 +993,14 @@ template <class T> T max (const matrix<T>& mat)
 }
 
 
-//--------------------------------------------
-template <class T> T tr (const matrix<T>& mat)
-//--------------------------------------------
+//-----------------------------------------------
+template <class Tf> Tf tr (const matrix<Tf>& mat)
+//-----------------------------------------------
 { 
   assert (mat.nbRows);
   assert (mat.nbRows==mat.nbColumns); // matrix must be squared
 
-  T val = 0;
+  Tf val = 0;
 
   for (int i=1; i<=mat.nbRows; i++)
     val += mat(i,i);
@@ -990,14 +1009,14 @@ template <class T> T tr (const matrix<T>& mat)
 }
 
 
-//----------------------------------------------------------
-template <class T> vector<T> diagonal (const matrix<T>& mat)
-//----------------------------------------------------------
+//-------------------------------------------------------------
+template <class Tf> vector<Tf> diagonal (const matrix<Tf>& mat)
+//-------------------------------------------------------------
 {
   assert (mat.nbRows);
   assert (mat.Rows()==mat.Columns());
 
-  vector<T> v(mat.nbRows,false);
+  vector<Tf> v(mat.nbRows,false);
 
   for (int i=1; i<=mat.nbRows; i++)
     v[i] = mat(i,i);
@@ -1006,14 +1025,14 @@ template <class T> vector<T> diagonal (const matrix<T>& mat)
 }
 
 
-//---------------------------------------------------------------------------------------
-template <class T> matrix<T> submatrix (const matrix<T>& mat, int m, int n, int p, int q)
-//---------------------------------------------------------------------------------------
+//------------------------------------------------------------------------------------------
+template <class Tf> matrix<Tf> submatrix (const matrix<Tf>& mat, int m, int n, int p, int q)
+//------------------------------------------------------------------------------------------
 {
   assert (m>0 && m<=n && n<=mat.nbRows);
   assert (p>0 && p<=q && q<=mat.nbColumns);
 
-  matrix<T> sub_mat;
+  matrix<Tf> sub_mat;
   sub_mat.create (n-m+1,q-p+1);
 
   for (int i=m; i<=n; i++)
@@ -1023,14 +1042,14 @@ template <class T> matrix<T> submatrix (const matrix<T>& mat, int m, int n, int
 }
 
 
-//------------------------------------------------------------------------------------
-template <class T> matrix<T> merge_rows (const matrix<T>& mat1, const matrix<T>& mat2)
-//------------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------------
+template <class Tf> matrix<Tf> merge_rows (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
+//----------------------------------------------------------------------------------------
 {
   assert (mat1.nbColumns==mat1[1].dim()); // excludes symmatrix
   assert (mat1.nbColumns == mat2.nbColumns);
 
-  matrix<T> mat;
+  matrix<Tf> mat;
   mat.create (mat1.nbRows+mat2.nbRows,mat1.nbColumns);
 
   for (int i=1; i<=mat1.nbRows; i++)
@@ -1042,14 +1061,14 @@ template <class T> matrix<T> merge_rows (const matrix<T>& mat1, const matrix<T>&
 }
 
 
-//---------------------------------------------------------------------------------------
-template <class T> matrix<T> merge_columns (const matrix<T>& mat1, const matrix<T>& mat2)
-//---------------------------------------------------------------------------------------
+//-------------------------------------------------------------------------------------------
+template <class Tf> matrix<Tf> merge_columns (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
+//-------------------------------------------------------------------------------------------
 {
   assert (mat1.nbColumns==mat1[1].dim()); // excludes symmatrix
   assert (mat1.nbRows == mat2.nbRows);
 
-  matrix<T> mat;
+  matrix<Tf> mat;
   mat.create (mat1.nbRows,mat1.nbColumns+mat2.nbColumns);
 
   for (int i=1; i<=mat.nbRows; i++)
