@@ -2,7 +2,7 @@
  *                 Mathematical Object Library                 *
  *      class spmatrix : declarations for sparse matrices      *
  *                    simula+@metz.ensam.fr                    *
- *		     GNU/linux version 0.0.8                   *
+ *		     GNU/linux version 0.0.9                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright  2004,2005 LAGOUCHE Sophie
@@ -42,7 +42,7 @@
              copyright \htmlonly &#169; \endhtmlonly 2004, 2005 Emmanuel CREUSE \n
 	     copyright \htmlonly &#169; \endhtmlonly 2004, 2005 Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	     copyright \htmlonly &#169; \endhtmlonly 2004, 2005 Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 0.0.8
+    \version 0.0.9
     \date 2004-2006
     \bug none
     \warning none
@@ -68,8 +68,8 @@
 #include <assert.h>
 #endif
 
-#if !defined(__PRECISION_H)
-#include "precision.h"
+#if !defined(__PARAMETERS_H)
+#include "../parameters.h"
 #endif
 
 #if !defined(__MATHS_H)
