@@ -2,7 +2,7 @@
  *                 Mathematical Object Library                 *
  *     class symmatrix:  declarations for symmetric matrix     *
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 1.3.4                   *
+ *                   GNU/linux version 1.4.6                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright  2002,2003,2004,2005,2006 COLLARD Christophe
@@ -42,7 +42,7 @@
              copyright \htmlonly &#169; \endhtmlonly 2002, 2003, 2004 Emmanuel CREUSE \n
 	     copyright \htmlonly &#169; \endhtmlonly 2002, 2003, 2004, 2005, 2006 Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	     copyright \htmlonly &#169; \endhtmlonly 2002, 2003, 2004, 2005, 2006 Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 1.3.4
+    \version 1.4.6
     \date 2002-2006
     \bug none
     \warning none
@@ -68,6 +68,10 @@
 #include <assert.h>
 #endif
 
+#if !defined(__PARAMETERS_H)
+#include "../parameters.h"
+#endif
+
 #if !defined(__VECTORS_H)
 #include "vectors.h"
 #endif
@@ -79,9 +83,9 @@
 using namespace std;
 
 
-//===================================================
-template <class T> class symmatrix : public matrix<T>
-//===================================================
+//===========================================================
+template <class T> class symmatrix : public virtual matrix<T>
+//===========================================================
 {
   using matrix<T>::nbRows;
   using matrix<T>::nbColumns;
@@ -101,26 +105,26 @@ template <class T> class symmatrix : public matrix<T>
     symmatrix<T>& operator = (const symmatrix<T>&);
     virtual symmatrix<T>& operator = (symmatrix<T>*);  // allocates the data of a temporary object to a real object (without copying data)
     symmatrix<T>& operator = (const matrix<T>&);
-    template <class M> friend symmatrix<M> operator + (const symmatrix<M>&, const symmatrix<M>&); // overloads plus operator for symmatrix
-    template <class M> friend symmatrix<M> operator - (const symmatrix<M>&, const symmatrix<M>&); // overloads plus operator for symmatrix
+    virtual symmatrix<T>& operator &= (const symmatrix<T>&);  // copies symmatrix without size check (use this operator with great care)
+    virtual symmatrix<T>& operator &= (symmatrix<T>*); // allocates the data from the right hand temporary object to the left hand object (deletes temporary object) - no size check - no copy (use this operator with great care)
+    template <class Tf> friend symmatrix<Tf> operator + (const symmatrix<Tf>&, const symmatrix<Tf>&); // overloads plus operator for symmatrix
+    template <class Tf> friend symmatrix<Tf> operator - (const symmatrix<Tf>&, const symmatrix<Tf>&); // overloads plus operator for symmatrix
     //    S * S' is not symmetric when S and S' are symmetric
-    template <class M> friend symmatrix<M> operator * (const symmatrix<M>&, const M&); // overloads multiplication for symmatrix & scalar
-    template <class M> friend symmatrix<M> operator * (const M&, const symmatrix<M>&); // overloads multiplication for scalar & symmatrix
-    template <class M> friend symmatrix<M> operator / (const symmatrix<M>&, const symmatrix<M>&); // overloads inversion for symmatrix (A OPTIMISER)
-    template <class M> friend symmatrix<M> operator / (const symmatrix<M>&, const M&); // overloads multiplication for symmatrix & scalar
-    template <class M> friend symmatrix<M> operator / (const M&, const symmatrix<M>&); // overloads multiplication for scalar & symmatrix
-    //    friend symmatrix<T> operator / <T> (const T&, const symmatrix<T>&); // overloads right multiplication between matrix and elements
+    template <class Tf> friend symmatrix<Tf> operator * (const symmatrix<Tf>&, const Tf&);  // overloads symmatrix * scalar
+    template <class Tf> friend symmatrix<Tf> operator * (const Tf&, const symmatrix<Tf>&);  // overloads scalar * symmatrix
+    //    S / S' is not symmetric when S and S' are symmetric
+    template <class Tf> friend symmatrix<Tf> operator / (const symmatrix<Tf>&, const Tf&);  // overloads symmatrix / scalar
+    template <class Tf> friend symmatrix<Tf> operator / (const Tf&, const symmatrix<Tf>&);  // overloads scalar / symmatrix
     void operator += (const symmatrix<T>&); // overloads += operator for symmatrix/symmatrix
     void operator += (const matrix<T>&);    // overloads += operator for symmatrix/matrix (which must be symmetric)
     void operator -= (const symmatrix<T>&); // overloads -= operator for symmatrix/symmatrix
     void operator -= (const matrix<T>&);    // overloads -= operator for symmatrix/matrix (which must be symmetric)
-    void operator /= (const symmatrix<T>&); // overloads /= operator for symmatrix
-    void operator /= (const T&); // overloads /= operator for symmatrix
-    template <class M> friend bool operator == (const symmatrix<M>&, const symmatrix<M>&); // optimizes == operator of symmatrix
-    template <class M> friend bool operator != (const symmatrix<M>&, const symmatrix<M>&); // same optimization
+    template <class Tf> friend bool operator == (const symmatrix<Tf>&, const symmatrix<Tf>&); // optimizes == operator of symmatrix
+    template <class Tf> friend bool operator != (const symmatrix<Tf>&, const symmatrix<Tf>&); // same optimization
     friend symmatrix<T> t (const symmatrix<T>& mat) {return mat;}        // optimizes tranposition operator for symmatrix
-    template <class M> friend symmatrix<M> gauss (const symmatrix<M>&);  // computes symmatrix inversion with gauss algorithm
-    template <class M> friend symmatrix<M> abs (const symmatrix<M>&);    // returns the symmatrix absolute value
+    template <class Tf> friend symmatrix<Tf> gauss (const symmatrix<Tf>&);  // computes symmatrix inversion with gauss algorithm
+    template <class Tf> friend symmatrix<Tf> abs (const symmatrix<Tf>&);    // returns the symmatrix absolute value
+    template <class Tf> friend symmatrix<Tf> Id2s (int);                    // returns identity matrix
 };
 
 
@@ -289,82 +293,116 @@ template <class T> symmatrix<T>& symmatrix<T>::operator = (symmatrix<T>* mat)
 }
 
 
-//---------------------------------------------------------------------------------------------
-template <class T> symmatrix<T> operator + (const symmatrix<T>& mat1, const symmatrix<T>& mat2)
-//---------------------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------
+template <class T> symmatrix<T>& symmatrix<T>::operator &= (const symmatrix<T>& mat)
+//----------------------------------------------------------------------------------
+{
+  if (!mat)
+    { if (nbRows) delete [] mrows;
+      nbRows = nbColumns = 0;
+    }
+
+  else
+    { assert (mat[1].dim()==1 && mat[mat.nbRows].dim()==mat.nbColumns); // forbids the acces to matrix && spmatrix
+      if (nbRows != mat.nbRows)
+	{ if (nbRows) delete [] mrows;
+	  mrows = new vector<T> [nbRows = mat.nbRows];  // allocates memory with the = operator for vector (no initialization)
+	  nbColumns = mat.nbColumns;
+	}
+      for (int i=1; i<=mat.nbRows; i++)
+	(*this)[i] = mat[i];
+    }
+
+  return *this;
+}
+
+
+//----------------------------------------------------------------------------
+template <class T> symmatrix<T>& symmatrix<T>::operator &= (symmatrix<T>* mat)
+//----------------------------------------------------------------------------
+{
+  if (!(*mat).nbRows)
+    { if (nbRows) delete [] mrows;
+      nbRows = nbColumns = 0;
+    }
+
+  else
+    { if (nbRows) delete [] mrows;
+      nbRows = (*mat).nbRows;
+      nbColumns = (*mat).nbColumns;
+      mrows = (*mat).mrows;
+      (*mat).nbRows = (*mat).nbColumns = 0;
+    }
+
+  return *this;
+}
+
+
+//-------------------------------------------------------------------------------------------------
+template <class Tf> symmatrix<Tf> operator + (const symmatrix<Tf>& mat1, const symmatrix<Tf>& mat2)
+//-------------------------------------------------------------------------------------------------
 {
   assert ( mat1.Rows()==mat2.Rows() ); // matrices must have the same size
   assert (mat1.Rows() && mat1.Columns());
-  symmatrix<T> mat = mat1;
+  symmatrix<Tf> mat = mat1;
   mat += mat2;
   return mat;
 }
 
 
-//---------------------------------------------------------------------------------------------
-template <class T> symmatrix<T> operator - (const symmatrix<T>& mat1, const symmatrix<T>& mat2)
-//---------------------------------------------------------------------------------------------
+//-------------------------------------------------------------------------------------------------
+template <class Tf> symmatrix<Tf> operator - (const symmatrix<Tf>& mat1, const symmatrix<Tf>& mat2)
+//-------------------------------------------------------------------------------------------------
 {
   assert ( mat1.Rows()==mat2.Rows() ); // matrices must have the same size
   assert (mat1.Rows() && mat1.Columns());
-  symmatrix<T> mat = mat1;
+  symmatrix<Tf> mat = mat1;
   mat -= mat2;
   return mat;
 }
 
 
-//--------------------------------------------------------------------------------
-template <class T> symmatrix<T> operator * (const symmatrix<T>& mat, const T& elt)
-//--------------------------------------------------------------------------------
+//------------------------------------------------------------------------------------
+template <class Tf> symmatrix<Tf> operator * (const symmatrix<Tf>& mat, const Tf& elt)
+//------------------------------------------------------------------------------------
 {
   assert (mat.nbRows && mat.nbColumns);
-  symmatrix<T> mp = mat;
+  symmatrix<Tf> mp = mat;
   mp *= elt;
   return mp;
 }
 
 
-//--------------------------------------------------------------------------------
-template <class T> symmatrix<T> operator * (const T& elt, const symmatrix<T>& mat)
-//--------------------------------------------------------------------------------
+//------------------------------------------------------------------------------------
+template <class Tf> symmatrix<Tf> operator * (const Tf& elt, const symmatrix<Tf>& mat)
+//------------------------------------------------------------------------------------
 {
   assert (mat.nbRows && mat.nbColumns);
-  symmatrix<T> mp = mat;
+  symmatrix<Tf> mp = mat;
   mp *= elt;
   return mp;
 }
 
 
-//---------------------------------------------------------------------------------------------
-template <class T> symmatrix<T> operator / (const symmatrix<T>& mat1, const symmatrix<T>& mat2)
-//---------------------------------------------------------------------------------------------
-{
-  assert (mat1.nbRows==mat2.nbRows);
-  symmatrix<T> mat = mat1;
-  mat /= mat2;
-  return mat;
-}
-
-
-//--------------------------------------------------------------------------------
-template <class T> symmatrix<T> operator / (const symmatrix<T>& mat, const T& elt)
-//--------------------------------------------------------------------------------
+//------------------------------------------------------------------------------------
+template <class Tf> symmatrix<Tf> operator / (const symmatrix<Tf>& mat, const Tf& elt)
+//------------------------------------------------------------------------------------
 {
   assert (mat.nbRows&&mat.nbColumns);
   assert (elt); // can't divide by zero
-  symmatrix<T> mp = mat;
+  symmatrix<Tf> mp = mat;
   mp /= elt;
   return mp;
 }
 
 
-//--------------------------------------------------------------------------------
-template <class T> symmatrix<T> operator / (const T& elt, const symmatrix<T>& mat)
-//--------------------------------------------------------------------------------
+//------------------------------------------------------------------------------------
+template <class Tf> symmatrix<Tf> operator / (const Tf& elt, const symmatrix<Tf>& mat)
+//------------------------------------------------------------------------------------
 {
   assert (mat.nbRows&&mat.nbColumns);
   assert (elt); // can't divide by zero
-  symmatrix<T> mp =& gauss(mat);
+  symmatrix<Tf> mp =& gauss(mat);
   mp *= elt;
   return mp;
 }
@@ -420,30 +458,9 @@ template <class T> void symmatrix<T>::operator -= (const matrix<T>& mat)
 }
 
 
-//-------------------------------------------------------------------------
-template <class T> void symmatrix<T>::operator /= (const symmatrix<T>& mat)
-//-------------------------------------------------------------------------
-{
-  assert (nbRows==mat.Rows()); // matrices must have the same size
-  // !!!!!!!!!!!!! optimiser l'operateur /= !!!!!!!!!!!!
-  (*this) = (*this) * gauss(mat);
-}
-
-
-//--------------------------------------------------------------
-template <class T> void symmatrix<T>::operator /= (const T& elt)
-//--------------------------------------------------------------
-{
-  assert (elt);
-
-  for (int i=1; i<=nbRows; i++)
-    (*this)[i] /= elt;
-}
-
-
-//--------------------------------------------------------------------------------------
-template <class T> bool operator == (const symmatrix<T>& mat1, const symmatrix<T>& mat2)
-//--------------------------------------------------------------------------------------
+//-----------------------------------------------------------------------------------------
+template <class Tf> bool operator == (const symmatrix<Tf>& mat1, const symmatrix<Tf>& mat2)
+//-----------------------------------------------------------------------------------------
 {
   bool result=( (mat1.nbRows==mat2.nbRows) && (mat2.nbColumns==mat2.nbColumns) );
   for (int i=1; i<=mat1.nbRows && result; i++)
@@ -452,33 +469,33 @@ template <class T> bool operator == (const symmatrix<T>& mat1, const symmatrix<T
 }
 
 
-//--------------------------------------------------------------------------------------
-template <class T> bool operator != (const symmatrix<T>& mat1, const symmatrix<T>& mat2)
-//--------------------------------------------------------------------------------------
+//-----------------------------------------------------------------------------------------
+template <class Tf> bool operator != (const symmatrix<Tf>& mat1, const symmatrix<Tf>& mat2)
+//-----------------------------------------------------------------------------------------
 {
   return !(mat1 == mat2);
 }
 
 
-//------------------------------------------------------------
-template <class T> symmatrix<T> gauss(const symmatrix<T>& mat)
-//------------------------------------------------------------
+//---------------------------------------------------------------
+template <class Tf> symmatrix<Tf> gauss(const symmatrix<Tf>& mat)
+//---------------------------------------------------------------
 {
   // optimiser imperativement l'algo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-  symmatrix<T> smt;
-  matrix<T> mt;
+  symmatrix<Tf> smt;
+  matrix<Tf> mt;
   mt = mat; // copie pour la convertion en trop
   smt = gauss(mt); // copie pour la convertion en trop + ADAPTER GAUSS
   return smt;
 }
 
 
-//-----------------------------------------------------------
-template <class T> symmatrix<T> abs (const symmatrix<T>& mat)
-//-----------------------------------------------------------
+//--------------------------------------------------------------
+template <class Tf> symmatrix<Tf> abs (const symmatrix<Tf>& mat)
+//--------------------------------------------------------------
 {
   assert (mat.nbColumns);
-  symmatrix<T> mtx;
+  symmatrix<Tf> mtx;
   mtx.create(mat.nbRows,mat.nbColumns);
 
   for (int i=1; i<=mat.nbRows; i++)
@@ -488,4 +505,18 @@ template <class T> symmatrix<T> abs (const symmatrix<T>& mat)
 }
 
 
+//--------------------------------------------
+template <class Tf> symmatrix<Tf> Id2s (int n)
+//--------------------------------------------
+{
+  assert (n > 0);
+  symmatrix<Tf> I(n,n);
+
+  for (int i=1; i<=n; i++)
+    I(i,i) = 1;
+
+  return I;
+}
+
+
 #endif
