@@ -2,7 +2,7 @@
  *                 Mathematical Object Library                 *
  *      class tensor3 : declarations for 3rd order tensors     *
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 0.4.6                   *
+ *                   GNU/linux version 0.5.2                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright  2004,2005,2006 COLLARD Christophe
@@ -39,7 +39,7 @@
     \author copyright \htmlonly &#169; \endhtmlonly 2004, 2005, 2006 Christophe COLLARD \n
 	    copyright \htmlonly &#169; \endhtmlonly 2004, 2005, 2006 Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	    copyright \htmlonly &#169; \endhtmlonly 2004, 2005, 2006 Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 0.4.6
+    \version 0.5.2
     \date 2004-2006
     \bug none
     \warning none
@@ -86,19 +86,20 @@ template <class T> class tensor3
     virtual int dim3 () const {return tsr[0].dim2();}    // returns the tensor 3rd component size : j
     virtual void create (int);          // partial memory allocation (don't use this function except if you know what you're doing !!)
     virtual void assign (int,int,int);  // allocates memory without initilizing the tensor components to zero (so use it with great care)
+    operator int () const {return size;}    // cast operator : tensor3 -> int
+    friend bool operator ! (const tensor3<T>& t) {return !t.size;}
 
     virtual T&  operator () (int,int,int) const;        // returns the element Pk,ij : P(tensor2,line,column)
     tensor2<T>& operator [] (int) const;                // returns a tensor2 Pk,ij -> Mij
     virtual tensor3<T>& operator = (const tensor3<T>&); // defines equal operator for 3rd order tensors
     virtual tensor3<T>& operator = (tensor3<T>*);  // allocates the data from the right hand temporary object to the left hand object - no copy (deletes temporary object)
-    virtual tensor3<T>& operator &= (tensor3<T>&); // allocates the data from the right hand member to the left one (deletes right hand member) - no size check - no copy (use this operator with great care)
+    virtual tensor3<T>& operator &= (const tensor3<T>&); // copies tensor3 without size check (use this operator with great care)
     virtual tensor3<T>& operator &= (tensor3<T>*); // allocates the data from the right hand temporary object to the left hand object (deletes temporary object) - no size check - no copy (use this operator with great care)
     template <class M> friend tensor3<M>  operator + (const tensor3<M>&, const tensor3<M>&); // overloads operator + for 3rd order tensors
     template <class M> friend tensor3<M>  operator - (const tensor3<M>&, const tensor3<M>&); // overloads operator - for 3rd order tensors
     template <class M> friend M           operator | (const tensor3<M>&, const tensor3<M>&);  // overloads operator | for 3rd order tensors T|P = Tijk Pijk
     tensor3<T>& operator += (const tensor3<T>&);  // overloads += operator for tensors
     tensor3<T>& operator -= (const tensor3<T>&);  // overloads -= operator for tensors
-    friend bool operator !  (const tensor3<T>& t) {return !t.size;}
     template <class M> friend bool operator == (const tensor3<M>&, const tensor3<M>&);  // compares 2 tensors
     template <class M> friend bool operator != (const tensor3<M>&, const tensor3<M>&);
     // operations with scalars
@@ -272,17 +273,22 @@ template <class T> tensor3<T>& tensor3<T>::operator = (tensor3<T>* tn)
 }
 
 
-//---------------------------------------------------------------------
-template <class T> tensor3<T>& tensor3<T>::operator &= (tensor3<T>& tn)
-//---------------------------------------------------------------------
+//---------------------------------------------------------------------------
+template <class T> tensor3<T>& tensor3<T>::operator &= (const tensor3<T>& tn)
+//---------------------------------------------------------------------------
 {
-  if (!tn.size) size = 0;
+  if (!tn)
+    { if (size) delete [] tsr;
+      size = 0;
+    }
+
   else
-    { if (size)
-	delete [] tsr;
-      size = tn.size;
-      tsr = tn.tsr;
-      tn.size = 0;
+    { if (size != tn.size)
+	{ if (size) delete [] tsr;
+	  tsr = new tensor2<T> [size = tn.size];  // allocates memory with the = operator for tensor2 (no initialization)
+	}
+      for (int i=1; i<=size; i++)
+	(*this)[i] &= tn[i];
     }
 
   return *this;
@@ -293,7 +299,11 @@ template <class T> tensor3<T>& tensor3<T>::operator &= (tensor3<T>& tn)
 template <class T> tensor3<T>& tensor3<T>::operator &= (tensor3<T>* tn)
 //---------------------------------------------------------------------
 {
-  if (!(*tn).size) size = 0;
+  if (!(*tn).size)
+    { if (size) delete [] tsr;
+      size = 0;
+    }
+
   else
     { if (size)
 	delete [] tsr;
