@@ -2,7 +2,7 @@
  *                 Mathematical Object Library                 *
  *      class tensor4 : declarations for 4th order tensors     *
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 0.5.2                   *
+ *                   GNU/linux version 0.6.0                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright  2004,2005,2006 COLLARD Christophe
@@ -39,7 +39,7 @@
     \author copyright \htmlonly &#169; \endhtmlonly 2004, 2005, 2006 Christophe COLLARD \n
 	    copyright \htmlonly &#169; \endhtmlonly 2004, 2005, 2006 Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	    copyright \htmlonly &#169; \endhtmlonly 2004, 2005, 2006 Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 0.5.2
+    \version 0.6.0
     \date 2004-2006
     \bug none
     \warning none
@@ -95,12 +95,14 @@ template <class T> class tensor4
     virtual int dim4 () const {return tsr[0].dim3();}    // returns the tensor 3rd component size : l
     virtual void create (int);              // partial memory allocation (don't use this function except if you know what you're doing !!)
     virtual void assign (int,int,int,int);  // allocates memory without initilizing the tensor components to zero
+    operator int () const {return size;}         // cast operator : tensor4 -> int
+    friend bool operator ! (const tensor4<T>& t) {return !t.size;}
 
     virtual T&   operator () (int,int,int,int) const;    // returns the element Pijkl : P(tensor3,tensor2,line,column)
     tensor3<T>& operator [] (int) const;                 // returns a 3rd order tensor Cijkl -> Tjkl
     virtual tensor4<T>& operator = (const tensor4<T>&);  // defines equal operator for 4th order tensors
     virtual tensor4<T>& operator = (tensor4<T>*);  // allocates the data from the right hand temporary object to the left hand object - no copy (deletes temporary object)
-    virtual tensor4<T>& operator &= (tensor4<T>&); // allocates the data from the right hand member to the left one (deletes right hand member) - no size check - no copy (use this operator with great care)
+    virtual tensor4<T>& operator &= (const tensor4<T>&); // copies tensor4 without size check (use this operator with great care)
     virtual tensor4<T>& operator &= (tensor4<T>*); // allocates the data from the right hand temporary object to the left hand object (deletes temporary object) - no size check - no copy (use this operator with great care)
     template <class M> friend tensor4<M> operator +  (const tensor4<M>&, const tensor4<M>&); // overloads operator + for 4th order tensors
     template <class M> friend tensor4<M> operator -  (const tensor4<M>&, const tensor4<M>&); // overloads operator - for 4th order tensors
@@ -108,7 +110,6 @@ template <class T> class tensor4
     template <class M> friend tensor4<M> operator || (const tensor4<M>&, const tensor4<M>&); // P_ijkl = T_ijpq R_pqkl
     tensor4<T>& operator += (const tensor4<T>&);     // overloads += operator for tensors
     tensor4<T>& operator -= (const tensor4<T>&);     // overloads -= operator for tensors
-    friend bool operator !  (const tensor4<T>& t) {return !t.size;}
     template <class M> friend bool operator == (const tensor4<M>&, const tensor4<M>&);  // compares 2 tensors
     template <class M> friend bool operator != (const tensor4<M>&, const tensor4<M>&);
     // operations with scalars
@@ -130,6 +131,8 @@ template <class T> class tensor4
     template <class M> friend tensor3<M> operator || (const tensor3<M>&, const tensor4<M>&);  // P_n,ij = R_n,kl T_klij
     template <class M> friend tensor3<M> operator || (const tensor4<M>&, const tensor3<M>&);  // P_n,ij = T_ijkl R_n,kl
 
+    template <class Tf> friend tensor4<Tf> Id4s (int);  // returns identity tensor for symmetric tensors : Tijkl = Tjikl = Tijlk
+
     template <class M> friend ostream& operator << (ostream&, const tensor4<M>&);  // overloads output stream for tensors
     template <class M> friend istream& operator >> (istream&, const tensor4<M>&);  // overloads output stream for tensors
     virtual void save (const char*); // writes to disk
@@ -284,17 +287,22 @@ template <class T> tensor4<T>& tensor4<T>::operator = (tensor4<T>* tn)
 }
 
 
-//---------------------------------------------------------------------
-template <class T> tensor4<T>& tensor4<T>::operator &= (tensor4<T>& tn)
-//---------------------------------------------------------------------
+//---------------------------------------------------------------------------
+template <class T> tensor4<T>& tensor4<T>::operator &= (const tensor4<T>& tn)
+//---------------------------------------------------------------------------
 {
-  if (!tn.size) size = 0;
+  if (!tn)
+    { if (size) delete [] tsr;
+      size = 0;
+    }
+
   else
-    { if (size)
-	delete [] tsr;
-      size = tn.size;
-      tsr = tn.tsr;
-      tn.size = 0;
+    { if (size != tn.size)
+	{ if (size) delete [] tsr;
+	  tsr = new tensor3<T> [size = tn.size];  // allocates memory with the = operator for tensor2 (no initialization)
+	}
+      for (int i=1; i<=size; i++)
+	(*this)[i] &= tn[i];
     }
 
   return *this;
@@ -305,7 +313,11 @@ template <class T> tensor4<T>& tensor4<T>::operator &= (tensor4<T>& tn)
 template <class T> tensor4<T>& tensor4<T>::operator &= (tensor4<T>* tn)
 //---------------------------------------------------------------------
 {
-  if (!(*tn).size) size = 0;
+  if (!tn)
+    { if (size) delete [] tsr;
+      size = 0;
+    }
+
   else
     { if (size)
 	delete [] tsr;
@@ -621,6 +633,23 @@ template <class T> tensor3<T> operator || (const tensor4<T>& t4, const tensor3<T
 }
 
 
+//------------------------------------------
+template <class Tf> tensor4<Tf> Id4s (int n)
+//------------------------------------------
+{
+  assert (n > 0);
+  tensor4<Tf> I(n,n,n,n);
+
+  for (int i=1; i<=n; i++)
+    for (int j=1; j<=n; j++)
+      { I(i,j,i,j) += .5;
+	I(i,j,j,i) += .5;
+      }
+
+  return I;
+}
+
+
 //------------------------------------------------------------------------
 template <class T> ostream& operator << (ostream& s, const tensor4<T>& tn)
 //------------------------------------------------------------------------
