@@ -2,7 +2,7 @@
  *                 Mathematical Object Library                 *
  *           class vector : declaration for vectors            *
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 1.6.8	               *
+ *	             GNU/linux version 1.7.8	               *
  *            software under General Public License            *
  ***************************************************************
  * copyright  1994,1995,1996,1999,2000,2001,2002,2003,2004,2005,2006 COLLARD Christophe
@@ -46,7 +46,7 @@
 	     copyright \htmlonly &#169; \endhtmlonly 2003, 2004 Denis MERCIER \n
 	     copyright \htmlonly &#169; \endhtmlonly 2002, 2003, 2004, 2005, 2006 Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	     copyright \htmlonly &#169; \endhtmlonly 2002, 2003, 2004, 2005, 2006 Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 1.6.8
+    \version 1.7.8
     \date 1994-2006
     \bug none
     \warning none
@@ -84,12 +84,12 @@
 #include <string>
 #endif
 
-#if !defined(__MATHS_H)
-#include "maths.h"
+#if !defined(__PARAMETERS_H)
+#include "../parameters.h"
 #endif
 
-#if !defined(__PRECISION_H)
-#include "precision.h"
+#if !defined(__MATHS_H)
+#include "maths.h"
 #endif
 
 using namespace std;
@@ -111,31 +111,33 @@ template <class T> class vector
     ~vector ();                     // destructor
 
     void assign (int);  // allocates memory for the vector but doen't initilize to zero its components (use it with great care)
-    int dim () const {return size;} // returns the vector size
+    int dim () const {return size;}       // returns the vector size
+    operator int () const {return size;}  // cast operator : vector -> int
+    friend bool operator ! (const vector<T>& v) {return !v.size;}
+
     T& operator [] (int) const;     // returns the address of an element
     virtual int operator = (int);   // allocates memory for the vector but doen't initilize to zero its components (use it with great care)
     virtual vector<T>& operator  = (const vector<T>&);
     virtual vector<T>& operator  = (vector<T>*); // allocates the data from the right hand temporary object to the left hand object - no copy (deletes temporary object)
-    virtual vector<T>& operator &= (vector<T>&); // allocates the data from the right hand member to the left one (deletes right hand member) - no size check - no copy (use this operator with great care)
+    virtual vector<T>& operator &= (const vector<T>&); // copies vector without size check (use this operator with great care)
     virtual vector<T>& operator &= (vector<T>*); // allocates the data from the right hand temporary object to the left hand object (deletes temporary object) - no size check - no copy (use this operator with great care)
-    friend bool operator ! (const vector<T>& v) {return !v.size;}
     template <class Tf> friend bool operator == (const vector<Tf>&, const vector<Tf>&);
     template <class Tf> friend bool operator != (const vector<Tf>&, const vector<Tf>&);
     template <class Tf> friend vector<Tf> operator + (const vector<Tf>&, const vector<Tf>&); // overloads operator + for vectors
     template <class Tf> friend vector<Tf> operator - (const vector<Tf>&, const vector<Tf>&); // overloads operator - for vectors
     template <class Tf> friend Tf         operator * (const vector<Tf>&, const vector<Tf>&); // defines the scalar product
-    template <class M> friend vector<M> operator / (const vector<M>&, const vector<M>&); // exists only if b.size=1
+    template <class Tf> friend vector<Tf> operator / (const vector<Tf>&, const vector<Tf>&); // exists only if b.size=1
     void operator += (const vector<T>&);
     void operator -= (const vector<T>&);
 // operations between scalars and vectors
-    template <class M> friend vector<M> operator * (const vector<M>&, const M&); // overloads vector * scalar operator
-    template <class M> friend vector<M> operator * (const M&, const vector<M>&); // overloads scalar * vector operator
-    template <class M> friend vector<M> operator / (const vector<M>&, const M&); // overloads vectors / scalars operator
+    template <class Tf> friend vector<Tf> operator * (const vector<Tf>&, const Tf&); // overloads vector * scalar operator
+    template <class Tf> friend vector<Tf> operator * (const Tf&, const vector<Tf>&); // overloads scalar * vector operator
+    template <class Tf> friend vector<Tf> operator / (const vector<Tf>&, const Tf&); // overloads vectors / scalars operator
     void operator *=  (const T&);
     void operator /=  (const T&);
 // overload for the outstream
-    template <class M> friend ostream& operator << (ostream&, const vector<M>&);
-    template <class M> friend istream& operator >> (istream&, const vector<M>&);
+    template <class Tf> friend ostream& operator << (ostream&, const vector<Tf>&);
+    template <class Tf> friend istream& operator >> (istream&, const vector<Tf>&);
 // i/o disk
     void save (const char*);
     void read (const char*);
@@ -145,11 +147,15 @@ template <class T> class vector
     vector<T>& approximation ();        // set to 0 an element < epsilon (machine precision)
     virtual vector<T> ortho2D();        // computes the orthogonal vector
     virtual void permute (int,int);     // elements permutation
-    template <class M> friend void permute (vector<M>&, vector<M>&);  // vectors permutation
-    template <class M> friend vector<M> abs (const vector<M>&);       // give the absolute value of a vector
-    template <class M> friend void order (vector<M>&);                // vectors ordering from smallest to biggest
-    template <class M> friend vector<M> subvector (const vector<M>&, int, int);     // extracts a sub-vector
-    template <class M> friend vector<M> merge (const vector<M>&, const vector<M>&); // merges 2 vectors
+    template <class Tf> friend void permute (vector<Tf>&, vector<Tf>&);  // vectors permutation
+    template <class Tf> friend vector<Tf> abs (const vector<Tf>&);       // give the absolute value of a vector
+    template <class Tf> friend void order (vector<Tf>&);                 // vectors ordering from smallest to biggest
+    template <class Tf> friend vector<Tf> subvector (const vector<Tf>&, int, int);      // extracts a sub-vector
+    template <class Tf> friend vector<Tf> merge (const vector<Tf>&, const vector<Tf>&); // merges 2 vectors
+
+// mpi functions
+    template <class Tf> friend void broadcast (vector<Tf>&, int);  // send vector on threads (see MPIOL++/broadcast.h)
+    template <class Tf> friend void secure_broadcast (vector<Tf>&, int); // send vector on threads (see MPIOL++/secure broadcast.h)
 };
 
 
@@ -385,9 +391,9 @@ template <class T> vector<T>& vector<T>::operator = (vector<T>* v)
   \f$ \textbf{w \&= y;} \; \textit{Deletes the table of coordinates of "w".} \f$ \n
 */
 
-//-----------------------------------------------------------------
-template <class T> vector<T>& vector<T>::operator &= (vector<T>& v)
-//-----------------------------------------------------------------
+//-----------------------------------------------------------------------
+template <class T> vector<T>& vector<T>::operator &= (const vector<T>& v)
+//-----------------------------------------------------------------------
 {
   if (!v.size)
     { if (size) delete [] p;
@@ -428,7 +434,11 @@ template <class T> vector<T>& vector<T>::operator &= (vector<T>& v)
 template <class T> vector<T>& vector<T>::operator &= (vector<T>* v)
 //-----------------------------------------------------------------
 {
-  if (!(*v).size) size=0;
+  if (!(*v).size)
+    { if (size) delete [] p;
+      size = 0;
+    }
+
   else
     { if (size) delete [] p;
       size = (*v).size; // copy the size of the temporary objet
@@ -576,13 +586,13 @@ template <class Tf> Tf operator * (const vector<Tf>& a, const vector<Tf>& b)
 }
 
 
-//------------------------------------------------------------------------------
-template <class T> vector<T> operator / (const vector<T>& a, const vector<T>& b)
-//------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------
+template <class Tf> vector<Tf> operator / (const vector<Tf>& a, const vector<Tf>& b)
+//----------------------------------------------------------------------------------
 {
   assert (a.size && (b.size==1)); // exists only if the size of b = 1
   assert (b.p[0]);
-  vector<T> v = a;
+  vector<Tf> v = a;
   v /= b.p[0];
   return v;
 }
@@ -610,32 +620,32 @@ template <class T> void vector<T>::operator -= (const vector<T>& v)
 }
 
 
-//----------------------------------------------------------------------
-template <class T> vector<T> operator * (const vector<T>& v, const T& s)
-//----------------------------------------------------------------------
+//--------------------------------------------------------------------------
+template <class Tf> vector<Tf> operator * (const vector<Tf>& v, const Tf& s)
+//--------------------------------------------------------------------------
 {
   assert (v.size);
-  vector<T> w = v;
+  vector<Tf> w = v;
   w *= s;
   return w;
 }
 
 
-//----------------------------------------------------------------------
-template <class T> vector<T> operator * (const T& s, const vector<T>& v)
-//----------------------------------------------------------------------
+//--------------------------------------------------------------------------
+template <class Tf> vector<Tf> operator * (const Tf& s, const vector<Tf>& v)
+//--------------------------------------------------------------------------
 {
   return v*s;
 }
 
 
-//----------------------------------------------------------------------
-template <class T> vector<T> operator / (const vector<T>& v, const T& s)
-//----------------------------------------------------------------------
+//--------------------------------------------------------------------------
+template <class Tf> vector<Tf> operator / (const vector<Tf>& v, const Tf& s)
+//--------------------------------------------------------------------------
 {
   assert (v.size);
   assert (abs(s) > epsilon); // can't divide by zero
-  vector<T> w = v;
+  vector<Tf> w = v;
   w /= s;
   return w;
 }
@@ -661,9 +671,9 @@ template <class T> void vector<T>::operator /= (const T& s)
 }
 
 
-//----------------------------------------------------------------------
-template <class T> ostream& operator << (ostream& s, const vector<T>& v)
-//----------------------------------------------------------------------
+//------------------------------------------------------------------------
+template <class Tf> ostream& operator << (ostream& s, const vector<Tf>& v)
+//------------------------------------------------------------------------
 {
   assert (v.size);
   for (int i=0; i<v.size; i++)
@@ -673,9 +683,9 @@ template <class T> ostream& operator << (ostream& s, const vector<T>& v)
 }
 
 
-//----------------------------------------------------------------------
-template <class T> istream& operator >> (istream& s, const vector<T>& v)
-//----------------------------------------------------------------------
+//------------------------------------------------------------------------
+template <class Tf> istream& operator >> (istream& s, const vector<Tf>& v)
+//------------------------------------------------------------------------
 {
   assert (v.size);
   for (int i=0; i<v.size; i++)
@@ -775,13 +785,13 @@ template <class T> void vector<T>::permute (int i, int j)
 }
 
 
-//----------------------------------------------------------
-template <class T> void permute (vector<T>& a, vector<T>& b)
-//----------------------------------------------------------
+//-------------------------------------------------------------
+template <class Tf> void permute (vector<Tf>& a, vector<Tf>& b)
+//-------------------------------------------------------------
 {
   assert (a.size);
   assert (a.size==b.size);
-  T* elt;
+  Tf* elt;
 
   elt = a.p;
   a.p = b.p;
@@ -789,12 +799,12 @@ template <class T> void permute (vector<T>& a, vector<T>& b)
 }
 
 
-//---------------------------------------------------
-template <class T> vector<T> abs (const vector<T>& v)
-//---------------------------------------------------
+//------------------------------------------------------
+template <class Tf> vector<Tf> abs (const vector<Tf>& v)
+//------------------------------------------------------
 { 
   assert (v.size);
-  vector<T> u(v.size,false);  // allocates memory without initializing the vector components to zero
+  vector<Tf> u(v.size,false);  // allocates memory without initializing the vector components to zero
 
   for (int i=0; i<v.size; i++)
     u.p[i] = abs(v.p[i]);
@@ -803,13 +813,13 @@ template <class T> vector<T> abs (const vector<T>& v)
 }
 
 
-//------------------------------------------
-template <class T> void order (vector<T>& v)
-//------------------------------------------
+//--------------------------------------------
+template <class Tf> void order (vector<Tf>& v)
+//--------------------------------------------
 {
   // A REVOIR et OPTIMISER
   assert (v.size);
-  T elt;
+  Tf elt;
 
   for (int i=1; i<v.size; i++)
     for (int j=i+1; j<=v.size; j++)
@@ -821,13 +831,13 @@ template <class T> void order (vector<T>& v)
 }
 
 
-//-----------------------------------------------------------------------
-template <class T> vector<T> subvector (const vector<T>& v, int i, int j)
-//-----------------------------------------------------------------------
+//--------------------------------------------------------------------------
+template <class Tf> vector<Tf> subvector (const vector<Tf>& v, int i, int j)
+//--------------------------------------------------------------------------
 {
   assert (v.size);
   assert (i>0 && i<=j && j<=v.size);
-  vector<T> u(j-i+1,false);
+  vector<Tf> u(j-i+1,false);
 
   for (int k=i; k<=j; k++)
     u.p[k-i] = v.p[k-1];
@@ -836,12 +846,12 @@ template <class T> vector<T> subvector (const vector<T>& v, int i, int j)
 }
 
 
-//-------------------------------------------------------------------------
-template <class T> vector<T> merge (const vector<T>& a, const vector<T>& b)
-//-------------------------------------------------------------------------
+//-----------------------------------------------------------------------------
+template <class Tf> vector<Tf> merge (const vector<Tf>& a, const vector<Tf>& b)
+//-----------------------------------------------------------------------------
 {
   assert (a.size && b.size);
-  vector<T> v (a.size + b.size,false);
+  vector<Tf> v (a.size + b.size,false);
 
   for (int i=0; i<a.size; i++)
     v.p[i] = a.p[i];
@@ -853,4 +863,3 @@ template <class T> vector<T> merge (const vector<T>& a, const vector<T>& b)
 
 
 #endif
-
