@@ -1,6 +1,6 @@
 /***************************************************************
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 0.3.2                   *
+ *	             GNU/linux version 0.3.8                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright  2004,2005,2006 COLLARD Christophe
@@ -64,7 +64,7 @@
 int test_tensor2 (int detail)
 //===========================
 {
-  int result=1;
+  bool result = true;
   vector<double> u(9), v(9), w(9);
 
   double reel=2.;
@@ -80,6 +80,14 @@ int test_tensor2 (int detail)
 
   tensor2<double> a(9,9) , c, d, e(9,9), f(9,9), g(9,9),h(9,9), Id(9,9);
 
+  bool test = true;
+  int size = 0;
+  if (a) size = a;
+  else test = false;
+  if (c) test = false;
+  if (detail) affiche ("cast operator () tensor2 -> int", test && size==a.dim1());
+  else result *= (test && size==a.dim1());
+
   if (detail) affiche ("operator !", !(!a) && !c);
   else result *= (!(!a) && !c);
 
@@ -139,14 +147,14 @@ int test_tensor2 (int detail)
   if (detail) affiche ("operator = (destructive copy for temporary objects)", mtx2==mtx3+mtx && mtx4==mtx3+mtx && mtx5==mtx+mtx3);
   else result *= (mtx2==mtx3+mtx && mtx4==mtx3+mtx && mtx5==mtx+mtx3);
 
-  tensor2<double> mtrx, mtrx2(2,3), mtrx3(12,24), mtrx4, mtrx5, mtrx6, mtrx7, mtrx8;
+  tensor2<double> mtrx, mtrx2(2,3), mtrx3(12,24), mtrx4, mtrx5(9,5), mtrx6, mtrx7, mtrx8;
   mtrx = mtrx8 = mtx;
   mtrx2 &= mtrx;
   mtrx3 &= mtrx4;
-  mtrx5 &= mtrx6;
+  mtrx5 &= nca;
   mtrx7 &= mtrx8;
-  if (detail) affiche ("operator &= (destructive copy with no size check)", mtrx2==mtx && !mtrx && !mtrx3 && !mtrx5 && mtrx7==mtx && !mtrx8);
-  else result *= (mtrx2==mtx && !mtrx && !mtrx3 && !mtrx5 && mtrx7==mtx && !mtrx8);
+  if (detail) affiche ("operator &= (copy without size check)", mtrx2==mtx && mtrx && !mtrx3 && mtrx7==mtx && nca==mtrx5 && mtrx5.dim2()==nca.dim2());
+  else result *= (mtrx2==mtx && mtrx && !mtrx3 && mtrx7==mtx && nca==mtrx5 && mtrx5.dim2()==nca.dim2());
 
   tensor2<double> mtrx9;
   mtrx = mtrx8 = mtx;
@@ -338,6 +346,12 @@ int test_tensor2 (int detail)
   if (detail) affiche ("gauss", (matx*gauss(matx) == matxId) && (a*gauss(a) == aId));
   else result *= ((matx*gauss(matx) == matxId) && (a*gauss(a) == aId));
 
+  tensor2<long double> Id_2(5,5,true,0);
+  for (int i=1; i<=5; i++)
+    Id_2(i,i) = 1;
+  if (detail) affiche("Id2", Id_2 == Id2<long double>(5));
+  else result *= (Id_2 == Id2<long double>(5));
+
   ncd.save("tensor2.res");
   tensor2<double> fmat(ncd.Rows(), ncd.Columns());
   fmat.read("tensor2.res");
