@@ -1,6 +1,6 @@
 /***************************************************************
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 1.3.2                   *
+ *	             GNU/linux version 1.3.8                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright  2003,2004,2005,2006 COLLARD Christophe
@@ -80,10 +80,18 @@
 int test_tensor4 (int detail)
 //===========================
 {
-  int result=1;
+  bool result = true;
 
   tensor4<long double> t1(3,5,7,2), t2(3,5,7,2), t3, t4(3,5,7,2), t5(3,5,7,2);
 
+  bool test = true;
+  int size = 0;
+  if (t1) size = t1;
+  else test = false;
+  if (t3) test = false;
+  if (detail) affiche ("cast operator () tensor4 -> int", test && size==t1.dim1());
+  else result *= (test && size==t1.dim1());
+
   if (detail) affiche ("operator !", !(!t1) && !t3);
   else result *= (!(!t1) && !t3);
 
@@ -161,21 +169,23 @@ int test_tensor4 (int detail)
   if (detail) affiche("operator = (destructive copy for temporary objects)", ntsr3==ntsr0 && ntsr3==ntsr4 && ntsr3==ntsr5);
   else result *= (ntsr3==ntsr0 && ntsr3==ntsr4 && ntsr3==ntsr5);
 
-  tensor4<long double> mtrx, mtrx2(2,3,5), mtrx3(12,24,36), mtrx4, mtrx5, mtrx6, mtrx7, mtrx8;
+  tensor4<long double> mtrx, mtrx2(2,3,5), mtrx3(12,24,36), mtrx4, mtrx5(2,9,5,12), mtrx6(2,9,11,12), mtrx0(2,3,11,12), mtrx7, mtrx8;
   mtrx = mtrx8 = ntsr1;
   mtrx2 &= mtrx;
   mtrx3 &= mtrx4;
-  mtrx5 &= mtrx6;
+  mtrx5 &= ntsr1;
+  mtrx6 &= ntsr1;
+  mtrx0 &= ntsr1;
   mtrx7 &= mtrx8;
-  if (detail) affiche ("operator &= (destructive copy with no size check)", mtrx2==ntsr1 && !mtrx && !mtrx3 && !mtrx5 && mtrx7==ntsr1 && !mtrx8);
-  else result *= (mtrx2==ntsr1 && !mtrx && !mtrx3 && !mtrx5 && mtrx7==ntsr1 && !mtrx8);
+  if (detail) affiche ("operator &= (copy without size check)", mtrx2==ntsr1 && mtrx && !mtrx3 && mtrx7==ntsr1 && mtrx5==ntsr1 && mtrx6==ntsr1 && mtrx0==ntsr1);
+  else result *= (mtrx2==ntsr1 && mtrx && !mtrx3 && mtrx7==ntsr1 && mtrx5==ntsr1 && mtrx6==ntsr1 && mtrx0==ntsr1);
 
-  tensor4<long double> mtrx9;
+  tensor4<long double> mtrx9, mtrx10;
   mtrx = mtrx8 = ntsr1;
   mtrx2 &=& mtrx;
   mtrx3 &=& (ntsr1+ntsr1);
   mtrx4 &=& mtrx;
-  mtrx5  &=& mtrx6;
+  mtrx5  &=& mtrx10;
   mtrx9 &=& mtrx8;
   if (detail) affiche ("operator &= (destructive copy for temporary objects with no size check)", mtrx2==ntsr1 && !mtrx && mtrx3==(long double) 2*ntsr1 && !mtrx4 && !mtrx5 && mtrx9==ntsr1 && !mtrx8);
   else result *= (mtrx2==ntsr1 && !mtrx && mtrx3==(long double) 2*ntsr1 && !mtrx4 && !mtrx5 && mtrx9==ntsr1 && !mtrx8);
@@ -365,6 +375,23 @@ int test_tensor4 (int detail)
   if (detail) affiche ("operator tensor4 || tensor3 :Tn,ij =  P_nkl Q_ijkl", t4t3 == (ntsr1 || ttest2));
   else result *= (t4t3 == (ntsr1 || ttest2));
 
+  int nsz = 3;
+  tensor4<long double> Id_4(nsz), Tsr4(nsz), Tsr4s(nsz);
+  for (int i=1; i<=nsz; i++)
+    for (int j=1; j<=nsz; j++)
+      for (int k=1; k<=nsz; k++)
+	for (int l=1; l<=nsz; l++)
+	  { Id_4(i,j,k,l) += .5 * ( (i==k) * (j==l) + (i==l) * (j==k));
+	    Tsr4(i,j,k,l) += .5*i + 2.134*j - 1.7654*k + 3.654298*l;
+	  }
+  for (int i=1; i<=nsz; i++)
+    for (int j=1; j<=nsz; j++)
+      for (int k=1; k<=nsz; k++)
+	for (int l=1; l<=nsz; l++)
+	  Tsr4s(i,j,k,l) = .25 * (Tsr4(i,j,k,l) + Tsr4(i,j,l,k) + Tsr4(j,i,k,l) + Tsr4(j,i,l,k));
+  if (detail) affiche("Id4s", Id_4==Id4s<long double>(3) && (Id4s<long double>(nsz) || Tsr4s)==Tsr4s && (Tsr4s || Id4s<long double>(nsz))==Tsr4s);
+  else result *= (Id_4==Id4s<long double>(3) && (Id4s<long double>(nsz) || Tsr4s)==Tsr4s && (Tsr4s || Id4s<long double>(nsz))==Tsr4s);
+
   t6.save("tensor4.res");
   tensor4<long double> ncd(t6.dim1(), t6.dim2(), t6.dim3(), t6.dim4());
   ncd.read("tensor4.res");
