@@ -1,6 +1,6 @@
 /***************************************************************
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 0.4.0                   *
+ *                   GNU/linux version 0.4.4                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright  2003,2004,2005,2006 COLLARD Christophe
@@ -84,6 +84,10 @@
 #include "tensors2-test.h"
 #endif
 
+#if !defined(__SYMTENSORS2_TEST_H)
+#include "symtensors2-test.h"
+#endif
+
 #if !defined(__TENSORS3_TEST_H)
 #include "tensors3-test.h"
 #endif
@@ -211,6 +215,18 @@ main()
 
   cout << endl;
 
+  detail = 0;
+  start = clock();
+  if (detail)
+    //=======================================================================
+    cout << "----------------2nd order symmetric tensors---------------- \n";
+    //=======================================================================
+  result *= test_symtensor2(detail);
+  end = clock();
+  cout<<"cpu time : " << (long double)(end-start)/CLOCKS_PER_SEC << endl;
+
+  cout << endl;
+
   detail = 0;
   start = clock();
   if (detail)
@@ -224,6 +240,7 @@ main()
   cout << endl;
 
   detail = 0;
+  start = clock();
   if (detail)
     //=======================================================================
     cout << "---------------------4th order tensor---------------------- \n";
