@@ -1,6 +1,6 @@
 /***************************************************************
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 1.4.2                   *
+ *                   GNU/linux version 1.4.4                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright  2003,2004,2005,2006 COLLARD Christophe
@@ -83,6 +83,14 @@ int test_vector (int detail)
   if (detail) affiche ("dim()", u.dim()==9);
   else result *= (u.dim()==9);
 
+  bool test = true;
+  int size = 0;
+  if (u) size = u;
+  else test = false;
+  if (x) test = false;
+  if (detail) affiche ("cast operator () vector -> int", test && size==u.dim());
+  else result *= (test && size==u.dim());
+
   if (detail) affiche ("operator !", !(!u) && !x);
   else result *= (!(!u) && !x);
 
